package server;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

/**
 * Třída demonstrující server pro komunikaci mezi programy pomocí
 * TCP.
 */
public class TCPServer {

    /**
     * Výstupní proud směrem ke klientovi.
     */
    private DataOutputStream output;
    /**
     * Vstupní proud směrem od klienta.
     */
    private DataInputStream input;
    /**
     * Serverový soket, na kterém server čeká na spojení od klienta.
     */
    private ServerSocket server;
    /**
     * Soket, pomocí kterého komunikuje server s konkrétním klientem.
     */
    private Socket connection;
    /**
     * Počet spojení.
     */
    private int counter = 1;
    /**
     * Port, na kterem server posloucha.
     */
    private int port;

    /**
     * Vytvoří novou instanci třídy TCPServer a inicializuje prvky
     * uživatelského rozhraní.
     */
    public TCPServer(int port) {
        this.port = port;
    }

    /**
     * Hlavní funkční metoda serveru, kde dochází k čekání a obsluze
     * spojení s klienty. Obsluha není prováděna vícevláknově, takže
     * je obsluhován pouze jeden klient najednou. Ostatní čekají ve
     * frontě na obsloužení.
     */
    public void runServer() {
        try {
            // vytvoření serverového soketu na portu 12345
            // s frontou délky 100 požadavků
            server = new ServerSocket(this.port, 100);
            while (true) {
                try {
                    // čekání na spojení
                    waitForConnection();
                    // získání vstupního a výstupního proudu
                    getStreams();
                    // zpracování spojení
                    processConnection();
                } catch (EOFException e) {
                    displayMessage("\nKlient ukončil spojení.");
                } finally {
                    // uzavření spojení
                    closeConnection();
                    counter++;
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Vyčká na spojení od klienta o po přijetí spojení zobrazí
     * zprávu.
     *
     * @throws IOException
     *             pokud dojde při čekání na spojení k chybě.
     */
    private void waitForConnection() throws IOException {
        displayMessage(String.format("Čekám na spojení na portu %d.\n", this.port));
        // blokující volání, čeká na spojení
        connection = server.accept();
        displayMessage("Spojení " + counter + " přijato od: "
                + connection.getInetAddress().getHostName());
    }

    /**
     * Získá ze soketu proudy pro odesílání a příjem dat.
     *
     * @throws IOException
     *             pokud dojde při získávání proudů k chybě.
     */
    private void getStreams() throws IOException {
        output = new DataOutputStream(connection.getOutputStream());
        input = new DataInputStream(connection.getInputStream());

        displayMessage("\nZískány vstupní a výstupní proud.\n");
    }

    /**
     * Zpracování spojení s klientem.
     *
     * @throws IOException
     *             pokud při komunikaci s klientem dojde k chybě.
     */
    private void processConnection() throws IOException {
        // zpracování zpráv od klienta
        String filename;

        // načtení zprávy
        // blokující čekání
        filename = input.readUTF();
        // zobrazení zprávy
        displayMessage("\nPožadován soubor: " + filename);
        File file = new File(filename);
        if(!file.canRead()) {
            output.write(0);
            output.flush();
            return;
        }
        FileInputStream finput;
        try {
            finput = new FileInputStream(file);
        } catch(Exception e) {
            output.write(0);
            output.flush();
            e.printStackTrace();
            return;
        }
        output.write(1);
        displayMessage("\nPosílám data");
        byte[] buffer  = new byte[512];
        int nacteno = -1;
        do {
            try {
               nacteno = finput.read(buffer);
               if(nacteno <= 0) break;
               output.write(buffer, 0, nacteno);
            } catch(Exception e) {
                output.flush();
                e.printStackTrace();
                return;
            }
        } while(true);
        output.flush();
    }

    /**
     * Uzavře proudy a soket.
     */
    private void closeConnection() {
        displayMessage("\nUzavírám spojení.\n");
        // zrušení možnosti editovat vstupní pole
        try {
            output.close();
            input.close();
            connection.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    /**
     * Přidá daný řetězec do textové oblasti. K úpravě dochází v
     * event dispatch vlákně.
     *
     * @param messageToDisplay
     *            zpráva, která se má přidat do textové oblasti.
     */
    private void displayMessage(final String messageToDisplay) {
        // final je tam proto, aby se k danému parametru dalo
        // přistoupit z metody vnitřní třídy
        System.out.print(messageToDisplay);
    }
}
